/** (C) 2002 Peter Stevens
 * This is licensed under the GNU GPL
 *
 * See the file copying for details
 */


/**
 dice.c
 
 This module implements a dice gadget
*/
#include "dice.h"

void DiceInit (FormPtr frm, UInt16 GadgetId) {

	DiceDataPtr dice = (DiceDataPtr)MemHandleNew(sizeof(DiceData));
	
	if (dice) {
		DiceData * p = (DiceData *)MemHandleLock((MemHandle)dice);
		(* p).value  = DiceRandom();
		(* p).status = 0;
		MemHandleUnlock((MemHandle)dice);
		if (frm) {
			FrmSetGadgetData(frm, FrmGetObjectIndex(frm,GadgetId), dice);
			DiceDraw(frm, GadgetId);
		}
	}
}

void DiceDestroy (FormPtr frm, UInt16 GadgetId) {

	MemHandle dice;
	
	dice = FrmGetGadgetData(frm, FrmGetObjectIndex(frm, GadgetId));
	if (dice) {
		MemHandleFree(dice);
	}
	
}

void DiceDraw(FormPtr frm, UInt16 GadgetId) {

	RectangleType bounds;
	Char text[1];
	UInt16 GadgetIndex = FrmGetObjectIndex(frm, GadgetId);
	FontID oldFontId;
	FrameBitsType frameBits;
	UInt16 status;
	
	//Grab the display value
	MemHandle data = FrmGetGadgetData(frm, FrmGetObjectIndex(frm,GadgetId));
	DiceData * p = (DiceData *)MemHandleLock(data);
	StrIToA (text, (*p).value);
	status = (*p).status;
	MemHandleUnlock(data);
	
	FrmGetObjectBounds(frm, GadgetIndex, &bounds);
	//Draw the frame
	frameBits.word = 0;
	frameBits.bits.shadowWidth = 0;
	frameBits.bits.width = 1;
	frameBits.bits.cornerDiam = 5;
	WinEraseRectangle(&bounds,0);
	WinDrawRectangleFrame(frameBits.word, &bounds);

	oldFontId = FntSetFont(ledFont);
	
	WinDrawChars ( text, StrLen(text), bounds.topLeft.x + 7, bounds.topLeft.y + 2);
	
	FntSetFont(oldFontId);
	if (status == 1) {
		WinInvertRectangle(&bounds, 0);
	}
}

Boolean DiceTapped(FormPtr frm, UInt16 GadgetId, EventPtr event) {

	UInt16 gadgetIndex = FrmGetObjectIndex(frm, GadgetId);
	RectangleType bounds;
	
	FrmGetObjectBounds(frm, gadgetIndex, &bounds);
	if (RctPtInRectangle (event->screenX, event->screenY, &bounds)) {
		DiceInvert(frm, GadgetId);
		return true;
	}
	return false;

}

void DiceInvert (FormPtr frm, UInt16 GadgetId){
	UInt16 gadgetIndex = FrmGetObjectIndex(frm, GadgetId);
	MemHandle data = FrmGetGadgetData(frm, gadgetIndex);
	DiceData * p = (DiceData *)MemHandleLock(data);
	if ((*p).status == 0) {
		(* p).status = 1;
	} 
	else {
		(*p).status = 0;
	}
	MemHandleUnlock(data);
	DiceDraw(frm, GadgetId);
}

/* This function rolls a new value for the dice and draws it if the dice is
 * selected (status 1)
 * */
void DiceRoll(FormPtr frm, UInt16 GadgetId) {

	UInt16 gadgetIndex = FrmGetObjectIndex(frm, GadgetId);
	MemHandle data = FrmGetGadgetData(frm, gadgetIndex);
	DiceData * p = (DiceData *)MemHandleLock(data);
	if ((*p).status == 1) {
		(* p).value = DiceRandom((*p).value);
	} 
	(*p).status = 0;
	MemHandleUnlock(data);
	DiceDraw(frm, GadgetId);	
}

/** This function rerolls all the dice
 */
void ReRoll() {
	int i;
	FormPtr frm = FrmGetActiveForm();

	for (i = 0; i < 5; i++) {
		UInt16 gadgetIndex = FrmGetObjectIndex(frm, Dice1  + i);
		MemHandle data = FrmGetGadgetData(frm, gadgetIndex);
		DiceData * p = (DiceData *)MemHandleLock(data);
		(* p).value = DiceRandom();
		(* p).status = 0;
		MemHandleUnlock(data);
		DrawAll();
	}
}


void DrawAll() {

	int i;
	FormPtr frm = FrmGetActiveForm();
	//updateLabel(Status,"Drawing");
	for (i = 0; i < 5; i++) {
		DiceDraw(frm, Dice1 + i);	
	}
	//updateLabel(Status,"Finished");
}

/** Generates a random number between 1 & 6 */
int DiceRandom() {

	return (SysRandom(0) % 6) + 1;
	
}



/** Returns an Int16[5] array with values corresponding to the dice displayed
 * onscreen
 * */

UInt16 die[];
void * getValues () {
	
	Int16 i;
	FormPtr frm = FrmGetActiveForm();
	for (i = 0; i < 5; i++) {

		//First lock the data
		MemHandle data = FrmGetGadgetData(frm, FrmGetObjectIndex(frm, Dice1+i));
		DiceData * p = (DiceData *)MemHandleLock(data);
		die[i] = (*p).value;
		MemHandleUnlock(data);
		
	}
	return die;
}

